# Implementation attachment

Reference implementation of game models with solving algorithms

## About

* Czech Technical University in Prague
* Faculty of Electrical Engineering
* Work: Bachelor's thesis
* Topic: Modeling Various Defense Actions in Adversarial Anomaly Detection Games
* Author: Martin Řepa
* Supervisor: Ing. Karel Durkota Ph.D.

#### Prerequisites

* Python 3.7
* pipenv

#### Structure

*Double oracle*
* `src/actors` contains both the defender and the attacker file with classes searching for best responses used in double oracle algorithm
* `src/neural/networks` is a module with implemented pytorch neural networks used as defender's actions
* `src/data` home for data used as records representing benign_users
* `src/visual` implements Debug plotter which plots summarization of each iteration of double oracle algorithm (must be enabled in `src/config.py`)
* `game.py` is entrypoint of double oracle approach. Takes RootConfig as input.
* `game_solver.py` implements double oracle and solves the game.
* `utility.py` definition of all utilty functions
* `config.py` configuration of all important variables which define the game

*Linear programming*
* `src/lp/lp_solver.py` implements solving of discretized game using linear programming. Small configuration can be found inside the file using global variables.



## Solving using double oracle 

#### Run locally
To run the source code locally follow these commands from root folder (same folder as this readme is located)

```
# First install dependencies using pipenv
pipenv install  

# To run example main with default config values specified in config.py
pipenv run python main.py

# !NOTE!
# By default double oracle approach does computations using GPU via cuda.
# To disable this feature, set env. variable DEVICE to 'cpu' before running
export DEVICE=cpu
```

#### Change values of game variables
To run the game with your own values of game variables you have two options:

* You can change default values of these variables inside `src/config.py` file
* Create similar python file as reference `main.py` and change values of RootConfig

All variables which you can update are documented in `src/config.py` file


## Solving using linear programming
Solver of discretized  game using linear programming is located in `src/lp/lp_solver.py` file.
Little configuration can be made using global variables inside this file.

To run the source code locally follow these commands from root folder (same folder as this readme is located)

```
# First install dependencies using pipenv (if you have not already done that)
pipenv install  

# Run lp python file from nested folder
export PYTHONPATH=$(pipenv --where) && pipenv run python src/lp/lp_solver.py
```
